<?php
/**
 * WooCommerce Price Robot - Products
 *
 * @version 1.1.1
 * @since   1.0.0
 * @author  Algoritmika Ltd.
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_Price_Robot_Products' ) ) :

class Alg_WC_Price_Robot_Products {

	/**
	 * Constructor.
	 *
	 * @version 1.1.0
	 * @since   1.0.0
	 */
	function __construct() {

		$this->is_wc_version_below_3 = version_compare( get_option( 'woocommerce_version', null ), '3.0.0', '<' );
		$this->price_filter = ( $this->is_wc_version_below_3 ) ? 'woocommerce_get_price' : 'woocommerce_product_get_price';

		// Price robot columns
		add_filter( 'manage_edit-product_columns',        array( $this, 'add_product_column_price_robot' ),    PHP_INT_MAX );
		add_action( 'manage_product_posts_custom_column', array( $this, 'render_product_column_price_robot' ), PHP_INT_MAX );

		// Filtering
		add_action( 'restrict_manage_posts', array( $this, 'restrict_manage_posts' ) );
		add_filter( 'parse_query',           array( $this, 'products_by_price_robot_admin_filter_query' ) );

		// Button
		add_action( 'admin_init', array( $this, 'enable_price_robot_for_product' ), PHP_INT_MAX );

	}

	/**
	 * Filter the products in admin based on options.
	 *
	 * @version 1.1.0
	 * @since   1.0.0
	 * @access  public
	 * @param   mixed $query
	 * @return  void
	 */
	function products_by_price_robot_admin_filter_query( $query ) {
		global $typenow, $wp_query;
		if ( $typenow == 'product' && isset( $_GET['price_robot'] ) && 'all' != $_GET['price_robot'] ) {
			if ( 'yes' === $_GET['price_robot'] ) {
				$query->query_vars['meta_value'] = 'yes';
				$query->query_vars['meta_key']   = '_price_robot_enabled';
			} elseif ( 'no' === $_GET['price_robot'] ) {
				$query->query_vars['meta_query'] = array(
					'relation' => 'OR',
					array(
						'key'     => '_price_robot_enabled',
						'compare' => 'NOT EXISTS',
						'value'   => '', // this is ignored, but is necessary...
					),
					array(
						'key'     => '_price_robot_enabled',
						'value'   => 'no',
					),
				);
			}
		}
	}

	/**
	 * Filters for post types.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function restrict_manage_posts() {
		global $typenow, $wp_query;
		if ( 'product' === $typenow ) {
			$selected_value = isset( $_GET['price_robot'] ) ? $_GET['price_robot'] : 'all';
			$values = array(
				'all' => __( 'Price Robot: All Products', 'price-robot-for-woocommerce' ),
				'yes' => __( 'Price Robot: Enabled', 'price-robot-for-woocommerce' ),
				'no'  => __( 'Price Robot: Disabled', 'price-robot-for-woocommerce' ),
			);
			echo '<select id="price_robot" name="price_robot">';
			foreach ( $values as $code => $name ) {
				echo '<option value="' . $code . '" ' . selected( $code, $selected_value, false ) . '>' . $name . '</option>';
			}
			echo '</select>';
		}
	}

	/**
	 * get_x.
	 *
	 * @version 1.1.0
	 * @since   1.0.0
	 */
	function get_x() {
		$args = array(
			'post_type'      => 'product',
			'post_status'    => 'any',
			'posts_per_page' => -1,
			'meta_key'       => '_price_robot_enabled',
			'meta_value'     => 'yes',
		);
		$loop = new WP_Query( $args );
		return $loop->found_posts;
	}

	/**
	 * enable_price_robot_for_product
	 *
	 * @version 1.1.1
	 * @since   1.0.0
	 */
	function enable_price_robot_for_product() {
		if ( isset( $_GET['enable_price_robot'] ) && 0 != $_GET['enable_price_robot'] ) {
			if ( $this->get_x() < apply_filters( 'alg_wc_price_robot', 10, 'value_products' ) ) {
				update_post_meta( $_GET['enable_price_robot'], '_price_robot_enabled', 'yes' );
			} else {
				echo '<div id="message" class="error"><p><strong>' . sprintf( __( 'Free plugin version is limited to ten price robot enabled products. Please get <a href="%s" target="_blank">Price Robot for WooCommerce Pro</a> to enable unlimited number of products.', 'price-robot-for-woocommerce' ), 'https://wpfactory.com/item/price-robot-for-woocommerce-plugin/' ) . '</strong></p></div>';
			}
		}
		if ( isset( $_GET['disable_price_robot'] ) && 0 != $_GET['disable_price_robot'] ) {
			update_post_meta( $_GET['disable_price_robot'], '_price_robot_enabled', 'no' );
		}
	}

	/**
	 * Add price robot column to products list.
	 *
	 * @version 1.1.0
	 * @since   1.0.0
	 */
	function add_product_column_price_robot( $columns ) {
		$columns['price_robot'] = __( 'Price Robot', 'price-robot-for-woocommerce' );
		if ( 'yes' === get_option( 'alg_price_robot_general_admin_products_debug_enabled', 'no' ) ) {
			$columns['price_robot_debug'] = __( 'Price Robot Debug', 'price-robot-for-woocommerce' );
		}
		return $columns;
	}

	/**
	 * render_product_column_price_robot.
	 *
	 * @version 1.1.0
	 */
	function render_product_column_price_robot( $column ) {

		if ( 'price_robot' != $column && 'price_robot_debug' != $column ) {
			return;
		}

		$column_content = '';

		$the_product_id = get_the_ID();
		$the_product    = wc_get_product( $the_product_id );

		if ( 'price_robot' === $column ) {
			$enabled = get_post_meta( $the_product_id, '_price_robot_enabled', true );
			if ( 'yes' === $enabled ) {
				$column_content .= '<a href="' . add_query_arg( 'disable_price_robot', $the_product_id, remove_query_arg( 'enable_price_robot' ) ) . '">'
					. '<img style="margin: -3px 3px;" width="16" height="16" title="' . __( 'Disable Price Robot', 'price-robot-for-woocommerce' ) . '" src="' .
						alg_wc_price_robot()->plugin_url() . '/assets/images/price-robot-green.png">' . '</a>';
			} else {
				$column_content .= '<a href="' . add_query_arg( 'enable_price_robot', $the_product_id, remove_query_arg( 'disable_price_robot' ) ) . '">'
					. '<img style="margin: -3px 3px;" width="16" height="16" title="' . __( 'Enable Price Robot', 'price-robot-for-woocommerce' ) . '" src="' .
						alg_wc_price_robot()->plugin_url() . '/assets/images/price-robot-gray.png">' . '</a>';
			}
		}

		$is_debug  = ( 'price_robot_debug' === $column );
		$separator = ( $is_debug ? '<br>' : ' ' );

		if ( 'no' === get_option( 'alg_price_robot_general_variable_as_single', 'no' ) && $the_product->is_type( 'variable' ) ) {
			$variations = $the_product->get_available_variations();
			foreach ( $variations as $variation ) {
				$variation_product = wc_get_product( $variation['variation_id'] );
				$column_content .= $this->get_price_robot_admin_html( $variation_product, $is_debug );
				$column_content .= $separator;
			}
		} else {
			$column_content .= $this->get_price_robot_admin_html( $the_product, $is_debug );
		}

		echo $column_content;
	}

	/**
	 * get_price_robot_admin_html
	 *
	 * @version 1.1.0
	 * @since   1.0.0
	 */
	function get_price_robot_admin_html( $_product, $debug_html = false ) {

		$the_product_id = alg_price_robot_get_product_id( $_product, $this->is_wc_version_below_3 );

		$price_robot_html = '';

		remove_filter( $this->price_filter, 'alg_price_robot', PHP_INT_MAX, 2 );
		$the_original_price = $_product->get_price();
		add_filter(    $this->price_filter, 'alg_price_robot', PHP_INT_MAX, 2 );

		$robot_price = alg_price_robot( $the_original_price, $_product, true );

		$color = ( $robot_price < $the_original_price ) ? 'red'   : 'gray';
		$color = ( $robot_price > $the_original_price ) ? 'green' : $color;

		$price_robot_html .= '<span style="color:' . $color . ';">' . wc_price( $robot_price ) . '</span>';

		if ( $debug_html ) {

			$last_sale       = get_post_meta( $the_product_id, '_last_sale',       true );
			$price_last_sale = get_post_meta( $the_product_id, '_price_last_sale', true );
			$no_sales = ( '' == $last_sale );
			if ( $no_sales ) {
				$last_sale       = get_post_time( 'U', false, $the_product_id ); // product creation time
				$price_last_sale = $the_original_price;
			}
			$price_last_sale = apply_filters( 'alg_price_robot_last_sale_price_last_sale', $price_last_sale, $last_sale, $no_sales );

			$price_robot_html .= '<br>' . sprintf( __( 'Last sale: %s', 'price-robot-for-woocommerce' ), date( 'Y-m-d H:i:s', $last_sale ) . ' @ ' . wc_price( $price_last_sale ) );
			$price_robot_html .= '<br>' . sprintf( __( 'Timeframe sales: %s', 'price-robot-for-woocommerce' ), get_post_meta( $the_product_id, '_timeframe_sales', true ) );
			$price_robot_html .= '<br>' . sprintf( __( 'Product ID: %s', 'price-robot-for-woocommerce' ), $the_product_id );
		}

		return $price_robot_html;
	}

}

endif;

return new Alg_WC_Price_Robot_Products();
