<?php
/*
Plugin Name: Price Robot for WooCommerce
Plugin URI: https://wpfactory.com/item/price-robot-for-woocommerce-plugin/
Description: The plugin calculates optimal price for products in WooCommerce.
Version: 1.1.1
Author: Algoritmika Ltd
Author URI: http://www.algoritmika.com
Text Domain: price-robot-for-woocommerce
Domain Path: /langs
Copyright:  2018 Algoritmika Ltd.
WC tested up to: 3.5
License: GNU General Public License v3.0
License URI: http://www.gnu.org/licenses/gpl-3.0.html
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

// Check if WooCommerce is active
if ( ! in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) return;

if ( 'price-robot-for-woocommerce.php' === basename( __FILE__ ) ) {
	// Check if Pro is active, if so then return
	$plugin = 'price-robot-for-woocommerce-pro/price-robot-for-woocommerce-pro.php';
	if (
		in_array( $plugin, apply_filters( 'active_plugins', get_option( 'active_plugins', array() ) ) ) ||
		( is_multisite() && array_key_exists( $plugin, get_site_option( 'active_sitewide_plugins', array() ) ) )
	) {
		return;
	}
}

if ( ! class_exists( 'Alg_Woocommerce_Price_Robot' ) ) :

/**
 * Main Alg_Woocommerce_Price_Robot Class
 *
 * @version 1.1.1
 * @class   Alg_Woocommerce_Price_Robot
 */
final class Alg_Woocommerce_Price_Robot {

	/**
	 * Plugin version.
	 *
	 * @var   string
	 * @since 1.1.0
	 */
	public $version = '1.1.1';

	/**
	 * @var Alg_Woocommerce_Price_Robot The single instance of the class
	 */
	protected static $_instance = null;

	/**
	 * Main Alg_Woocommerce_Price_Robot Instance
	 *
	 * Ensures only one instance of Alg_Woocommerce_Price_Robot is loaded or can be loaded.
	 *
	 * @static
	 * @return Alg_Woocommerce_Price_Robot - Main instance
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	/**
	 * Alg_Woocommerce_Price_Robot Constructor.
	 *
	 * @version 1.1.0
	 * @access  public
	 * @todo    (maybe) check Zendesk ticket for robot rules
	 * @todo    (maybe) by product pageviews
	 */
	function __construct() {

		// Set up localisation
		load_plugin_textdomain( 'price-robot-for-woocommerce', false, dirname( plugin_basename( __FILE__ ) ) . '/langs/' );

		// Include required files
		$this->includes();

		// Settings
		if ( is_admin() ) {
			add_filter( 'woocommerce_get_settings_pages', array( $this, 'add_woocommerce_settings_tab' ) );
			add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $this, 'action_links' ) );
		}
	}

	/**
	 * Show action links on the plugin screen.
	 *
	 * @version 1.1.1
	 * @param   mixed $links
	 * @return  array
	 */
	function action_links( $links ) {
		$custom_links = array();
		$custom_links[] = '<a href="' . admin_url( 'admin.php?page=wc-settings&tab=alg_price_robot' ) . '">' . __( 'Settings', 'woocommerce' ) . '</a>';
		if ( 'price-robot-for-woocommerce.php' === basename( __FILE__ ) ) {
			$custom_links[] = '<a href="https://wpfactory.com/item/price-robot-for-woocommerce-plugin/">' . __( 'Unlock All', 'price-robot-for-woocommerce' ) . '</a>';
		}
		return array_merge( $custom_links, $links );
	}

	/**
	 * Include required core files used in admin and on the frontend.
	 *
	 * @version 1.1.0
	 */
	function includes() {

		require_once( 'includes/price-robot-functions.php' );
		require_once( 'includes/robots/class-wc-price-robot-abstract.php' );

		$settings = array();

		// Settings
		$settings[] = require_once( 'includes/admin/class-wc-price-robot-settings-general.php' );

		// Robots
		$settings[] = require_once( 'includes/robots/class-wc-price-robot-last-sale.php' );
		$settings[] = require_once( 'includes/robots/class-wc-price-robot-pretty-price.php' );

		if ( is_admin() && get_option( 'alg_wc_price_robot_version', '' ) !== $this->version ) {
			foreach ( $settings as $section ) {
				foreach ( $section->get_settings() as $value ) {
					if ( isset( $value['default'] ) && isset( $value['id'] ) ) {
						$autoload = isset( $value['autoload'] ) ? ( bool ) $value['autoload'] : true;
						add_option( $value['id'], $value['default'], '', ( $autoload ? 'yes' : 'no' ) );
					}
				}
				update_option( 'alg_wc_price_robot_version', $this->version );
			}
		}

		require_once( 'includes/admin/class-wc-price-robot-products.php' );
		require_once( 'includes/admin/class-wc-price-robot-crons.php' );
		require_once( 'includes/class-wc-price-robot.php' );
	}

	/**
	 * Add Woocommerce settings tab to WooCommerce settings.
	 *
	 * @version 1.1.1
	 */
	function add_woocommerce_settings_tab( $settings ) {
		$settings[] = require_once( 'includes/admin/class-wc-settings-price-robot.php' );
		return $settings;
	}

	/**
	 * Get the plugin url.
	 *
	 * @return string
	 */
	function plugin_url() {
		return untrailingslashit( plugin_dir_url( __FILE__ ) );
	}

	/**
	 * Get the plugin path.
	 *
	 * @return string
	 */
	function plugin_path() {
		return untrailingslashit( plugin_dir_path( __FILE__ ) );
	}

}

endif;

if ( ! function_exists( 'alg_wc_price_robot' ) ) {
	/**
	 * Returns the main instance of Alg_Woocommerce_Price_Robot to prevent the need to use globals.
	 *
	 * @return Alg_Woocommerce_Price_Robot
	 */
	function alg_wc_price_robot() {
		return Alg_Woocommerce_Price_Robot::instance();
	}
}

alg_wc_price_robot();
